#!/usr/bin/env bash

########################
# Variables

# Note: The repetition of tile types in the tiles list weights repeated tile types to appear more frequently
declare    tiles="gggggpppphhff" IFS=$'\n' zeroPad="0000"
declare -a newLines terrainLayer riverLayer e00Layer e01Layer e02Layer resLayer island islandA islandB islandC islandList=(1 2 3 4 5)
declare -i lineCount number mapWidth=50 mapHeight=50 islandCount startPosCount islandsAcross islandsHigh spacePadding selectedIsland=1



########################
# Usage
usage="Usage:	$(basename ${0}) [-w width] [-h height] [-i island 1 through ${#islandList[*]} ]\n\
	width is width in tiles. Default value: 100\n\
	height is height in tiles. Default value: 50\n"

until [ -z "${1}" ] ; do
	case "${1}" in
		-x | -X | -w | -W | --width		)	shift ; mapWidth="${1}" ;;
		-y | -Y | -h | -H | --height		)	shift ; mapHeight="${1}" ;;
		-i | -I | --island			)	shift ; selectedIsland="${1}" ;;
		--help 					)	printf "${usage}" ; exit 1 ;;
		* 					)	printf "${usage}" ; exit 1 ;;
	esac
	shift
done


if [ ${mapWidth} -lt 16 -o ${mapHeight} -lt 16 ] ; then
	echo "FreeCiv minimum map width/height is 16"
	exit 1
fi

if [ ${selectedIsland} -lt 1 -o ${selectedIsland} -gt ${#islandList[*]} ] ; then
	echo "Unrecognized island pattern designation ${selectedIsland}"
	echo "1 through ${#islandList[*]} are valid choices"
	exit 1
fi



########################
# Functions
GetTilesNumber() {
	let "number = ${RANDOM} % ${#tiles}"
}

AddIsland() {
	x=${1} y=$((${2}+${spacePadding}))
	local -i k l
	for ((k=${y},l=0;k<$((${2}+${islandHeight}+2));k++,l++)) ; do
		terrainLayer[${k}]="${terrainLayer[${k}]:0:$(( ${x}+${spacePadding} ))}${island[${l}]}${terrainLayer[${k}]:$(( ${x} + ${islandWidth} + ${spacePadding} )) }"
	done
	islandCount=$((${islandCount}+1))

	# Set startposition for inner islands
	startPosOutput="${startPosOutput}$((${1}+8)),$((${2}+8)),FALSE,\"\"${IFS}"
	startPosCount=$((${startPosCount}+1))

	# Set startposition for outer islands if the outer ring is NOT arctic tiles
	# If the outer island is arctic tiles, no guards are needed
	if [ "${terrainLayer[2]:2:1}" != "a" ] ; then
		startPosOutput="${startPosOutput}$((${1}+2)),$((${2}+2)),FALSE,\"\"${IFS}"
		startPosCount=$((${startPosCount}+1))
	fi
}

LoadIsland() {
	local whichIsland="${1}"
	if   [ ${whichIsland} = 1 ] ; then
		island=( ${island1[*]} )
	elif [ ${whichIsland} = 2 ] ; then
		island=( ${island2[*]} )
	elif [ ${whichIsland} = 3 ] ; then
		island=( ${island3[*]} )
	elif [ ${whichIsland} = 4 ] ; then
		island=( ${island4[*]} )
	else
		island=( ${island5[*]} )
	fi

	islandWidth=${#island[0]}
	islandHeight=${#island[*]}
}


########################
# Island patterns

# island arrays hold the island patterns
# Width is 9, height is 11
# outer island has 36 tiles
# inner island has 35 tiles
 island1[0]="XXXXXXXXX"
 island1[1]="X       X"
 island1[2]="X XXXXX X"
 island1[3]="X XXXXX X"
 island1[4]="X XXXXX X"
 island1[5]="X XXXXX X"
 island1[6]="X XXXXX X"
 island1[7]="X XXXXX X"
 island1[8]="X XXXXX X"
 island1[9]="X       X"
island1[10]="XXXXXXXXX"

# Width is 11, height is 12
# outer island has 44 tiles
# inner island has 52 tiles
 island2[0]="XXXXXXXXXXX"
 island2[1]="XX       XX"
 island2[2]="X  XXXXX  X"
 island2[3]="X XXXXXXX X"
 island2[4]="X XXXXXXX X"
 island2[5]="X XXXXXXX X"
 island2[6]="X XXXXXXX X"
 island2[7]="X XXXXXXX X"
 island2[8]="X XXXXXXX X"
 island2[9]="X  XXXXX  X"
island2[10]="XX       XX"
island2[11]="XXXXXXXXXXX"

# Width is 11, height is 9
# outer island has 44 tiles
# inner island has 36 tiles
 island3[0]="XXXXXXXXXXX"
 island3[1]="X         X"
 island3[2]="X XXXXXXX X"
 island3[3]="X XXXXXXX X"
 island3[4]="X XXXXXXX X"
 island3[5]="X XXXXXXX X"
 island3[6]="X XXXXXXX X"
 island3[7]="X         X"
 island3[8]="XXXXXXXXXXX"

# Width is 11, height is 12
# outer island has 52 tiles
# inner island has 42 tiles
 island4[0]="XXXXXXXXXXX"
 island4[1]="XX       XX"
 island4[2]="XX XXXXX XX"
 island4[3]="XX XXXXX XX"
 island4[4]="X  XXXXX  X"
 island4[5]="X XXXXXXX X"
 island4[6]="X  XXXXX  X"
 island4[7]="XX XXXXX XX"
 island4[8]="XX XXXXX XX"
 island4[9]="XX XXXXX XX"
island4[10]="XX       XX"
island4[11]="XXXXXXXXXXX"

# Width is 11, height is 12
# outer island has 52 tiles
# inner island has 42 tiles
 island5[0]="aaaaaaaaaaa"
 island5[1]="aa       aa"
 island5[2]="aa XXXXX aa"
 island5[3]="aa XXXXX aa"
 island5[4]="a  XXXXX  a"
 island5[5]="a XXXXXXX a"
 island5[6]="a XXXXXXX a"
 island5[7]="a  XXXXX  a"
 island5[8]="aa XXXXX aa"
 island5[9]="aa XXXXX aa"
island5[10]="aa       aa"
island5[11]="aaaaaaaaaaa"
# The "a" (arctic/glacier) tiles used in pattern5 will stay as they are.
# Arctic tiles prevent settlers from building, thus no guards are needed on this island.
# The ice itself imprisons the inner island's tribe.
# Escape (or access from outside) will only be possible once Engineers are available to transform the tiles,
#  or if an ally brings a boat to the ice ring, but they too would have to have engineers to escape their island!
# This method delays combat until -after- the Explosives technology has been researched.



########################
# Generate the map

LoadIsland "${selectedIsland}"

spacePadding=2

 mapWidth=$(( ( ${mapWidth}  / ( ${islandWidth}  + (${spacePadding}+2) ) ) * ( ${islandWidth} +  (${spacePadding}+2) ) ))
mapHeight=$(( ( ${mapHeight} / ( ${islandHeight} + (${spacePadding}+2) ) ) * ( ${islandHeight} + (${spacePadding}+2) ) ))

islandsAcross=$(( ${mapWidth} / ( ${islandWidth} + (${spacePadding}*2) ) ))
  islandsHigh=$(( ${mapHeight} / ( ${islandHeight} + (${spacePadding}*2) ) ))

# Create the map layer arrays
terrainLayer[0]=$(printf "%*s" ${mapWidth})
resLayer[0]=$(printf "%*s" ${mapWidth})
riverLayer[0]=$(temp=$(printf "%*s" ${mapWidth}) ; echo "${temp// /0}")
e00Layer[0]="${riverLayer[0]}"
e01Layer[0]="${riverLayer[0]}"
e02Layer[0]="${riverLayer[0]}"

for ((i=1;i<${mapHeight};i++)) ; do
	terrainLayer[${i}]="${terrainLayer[0]}"
	e00Layer[${i}]="${riverLayer[0]}"
	e01Layer[${i}]="${riverLayer[0]}"
	e02Layer[${i}]="${riverLayer[0]}"
	riverLayer[${i}]="${riverLayer[0]}"
	resLayer[${i}]="${resLayer[0]}"
done

# Add islands to the terrain layer map
for ((i=0;i<${islandsHigh};i++)) ; do
	for ((j=0;j<${islandsAcross};j++)) ; do
		AddIsland $(( ${j} * ( ${islandWidth} + (${spacePadding}*2) ) )) $(( ${i} * ( ${islandHeight} + (${spacePadding}*2) )  ))
	done
done



########################
# Randomize the land tiles (character X is the placeholder for land tiles)

for ((i=0;i<${#terrainLayer[*]};i++)) ; do
	for ((j=0;j<${#terrainLayer[${i}]};j++)) ; do
		# If tile should be land
		if [ "${terrainLayer[${i}]:${j}:1}" = "X" ] ; then
			GetTilesNumber
			# If the preceeding chr of this line is the same as the new chr, pick again, one retry only.
			if [ "${newLines[${i}]:$((${j}-1)):1}" = "${tiles:${number}:1}" ] ; then
				GetTilesNumber
			fi
			newLines[${i}]="${newLines[${i}]}${tiles:${number}:1}"

			# If tile is -not- lake (+)
			if [ "${newLines[${i}]:${j}:1}" != "+" ] ; then
				# 1 in 10 chance of river on tile which is 10% of the land tiles
				if [ $(( ${RANDOM} % 10 )) -eq 0 -a $((${j}+1)) -le ${mapWidth} ] ; then
					riverLayer[${i}]="${riverLayer[${i}]:0:${j}}2${riverLayer[${i}]:$((${j}+1))}"

					# Fix for when the tile is the last one on the line
					[ ${#riverLayer[${i}]} -gt ${mapWidth} ] && riverLayer[${i}]="${riverLayer[${i}]:0:${mapWidth}}"
				fi
			fi
		else
			newLines[${i}]="${newLines[${i}]}${terrainLayer[${i}]:${j}:1}"
		fi
	done
done

terrainLayer=(${newLines[*]})



########################
# Output

# terrain layer
for ((i=0;i<${mapHeight};i++)) ; do
	[ "${terrainLayer[${i}]}" != "" ] && echo "t${zeroPad:${#i}}${i}=\"${terrainLayer[${i}]}\"" && lineCount=$((${lineCount}+1))
done

# start positions list
echo "startpos_count=${startPosCount}"
echo "startpos={\"x\",\"y\",\"exclude\",\"nations\""
echo -n "${startPosOutput}"
echo '}'

# e00_ Irrigation, mine, pollution, oil wells layer
for ((i=0;i<${lineCount};i++)) ; do
	echo "e00_${zeroPad:${#i}}${i}=\"${e00Layer[${i}]:0:${#terrainLayer[0]}}\""
done

# e01_ Hut, farmland, fallout, fortress layer
for ((i=0;i<${lineCount};i++)) ; do
	echo "e01_${zeroPad:${#i}}${i}=\"${e01Layer[${i}]:0:${#terrainLayer[0]}}\""
done

# e02_ Rail, airbase, buoy, ruins layer
for ((i=0;i<${lineCount};i++)) ; do
	echo "e02_${zeroPad:${#i}}${i}=\"${e02Layer[${i}]:0:${#terrainLayer[0]}}\""
done

# e03_ River and road layer
for ((i=0;i<${lineCount};i++)) ; do
	echo "e03_${zeroPad:${#i}}${i}=\"${riverLayer[${i}]:0:${#terrainLayer[0]}}\""
done

# res (resources) layer
for ((i=0;i<${lineCount};i++)) ; do
	echo "res${zeroPad:${#i}}${i}=\"${resLayer[${i}]:0:${#terrainLayer[0]}}\""
done

echo
echo "    TOTAL ISLAND SETS: ${islandCount}  (${islandsAcross} X ${islandsHigh})"
echo " ACTUAL WIDTH / xsize: ${#terrainLayer[0]}"
echo "ACTUAL HEIGHT / ysize: ${lineCount}"
echo "       startpos_count: ${startPosCount} ( <-- Set "aifill" and "maxplayers" to this number!)"

exit 0



